<?php

namespace Form;

use Form\Constraint\Constraint;
use Symfony\Component\HttpFoundation\Request;

class Field
{

    /**
     * Input name
     * @var string
     */
    protected $name;

    /**
     * @var array
     */
    protected $messages;

    /**
     * @var Constraint[]
     */
    protected $constraints;

    public function __construct($name, $constraints = array(), $messages = array())
    {
        $this->name = $name;
        $this->constraints = $constraints;
        $this->messages = $messages;
    }

    public function addConstraints($name, Constraint $constraint, $message = '')
    {
        $this->constraints[$name] = $constraint;
        $this->messages[$name] = $message;

        return $this;
    }

    public function addMessage($name, $message)
    {
        $this->messages[$name] = $message;
        return $this;
    }

    public function getName()
    {
        return $this->name;
    }

    public function isValid(Request $request)
    {
        $errors = array();
        foreach ($this->constraints as $name => $constraint) {
            if (in_array('Form\\RequestAwareInterface', class_implements($constraint))) {
                $constraint->setRequest($request);
            }

            $result = $constraint->isValid($request->get($this->getName()));

            if ($result == false) {
                $errors[] = (!empty($this->messages[$name])) ? $this->messages[$name] : $name;
            }
        }

        return $errors;
    }
}
